#include "Ifx_Types.h"
#include "IfxGtm_Atom_PwmHl.h"
#include "IfxGtm_Atom_Dtm_PwmHl.h"
#include "PWM_Atom.h"


// Configuration
#define FXUCLK_FREQUENCY        100e6                                       // FXU Clock base frequency, in Hertz
#define PWM_FREQUENCY           105e3                                       // PWM signal frequency, in Hertz
#define PWM_PERIOD              (FXUCLK_FREQUENCY / PWM_FREQUENCY)          // PWM signal period, in ticks
//#define DEADTIME                200e-9                                      // Dead time in seconds - 200 ns. When an
                                                                            // unexpected deadtime of 2.55e-x occurs,
                                                                            // the deadtime is bigger than allowed.

#define NUMBER_OF_CHANNELS      1
#define LED
//#define OSCILLOSCOPE
//
//#define CHANNEL1_DUTY_CYCLE     25                                          // PWM Signal 1 Duty cycle, in percent
//#define CHANNEL2_DUTY_CYCLE     50                                          // PWM Signal 2 Duty cycle, in percent
//#define CHANNEL3_DUTY_CYCLE     75                                          // PWM Signal 3 Duty cycle, in percent
//
//#define CHANNEL1_COMPARE_VALUE  ((PWM_PERIOD / 100) * (100 - CHANNEL1_DUTY_CYCLE))
//#define CHANNEL2_COMPARE_VALUE  ((PWM_PERIOD / 100) * (100 - CHANNEL2_DUTY_CYCLE))
//#define CHANNEL3_COMPARE_VALUE  ((PWM_PERIOD / 100) * (100 - CHANNEL3_DUTY_CYCLE))

void init_stdif_pwmhl(void)
{
    // prepare ccx coutx in advance
    IfxGtm_Atom_ToutMapP ccx[NUMBER_OF_CHANNELS];
    IfxGtm_Atom_ToutMapP coutx[NUMBER_OF_CHANNELS];
//    coutx[0] // todo(abemat): verify pin, will it work in combination with chosen ccx pin?
    uint8 iCcx = 0;
#ifdef OSCILLOSCOPE
    ccx[iCcx] = &IfxGtm_ATOM0_0_TOUT0_P02_0_OUT;
    coutx[iCcx] = &IfxGtm_ATOM0_1_TOUT1_P02_1_OUT;
    iCcx++;
#elif defined( LED )
    ccx[iCcx] = &IfxGtm_ATOM2_5_TOUT91_P13_0_OUT;
    coutx[iCcx] = &IfxGtm_ATOM2_6_TOUT92_P13_1_OUT;
#endif
//    IfxGtm_Atom_ToutMapP ccx[3] = {&IfxGtm_ATOM0_0_TOUT0_P02_0_OUT, &IfxGtm_ATOM0_1_TOUT1_P02_1_OUT, &IfxGtm_ATOM0_2_TOUT2_P02_2_OUT};
//    IfxGtm_Atom_ToutMapP ccx[] = {IfxGtm_ATOM0_0_TOUT0_P02_0_OUT, IfxGtm_ATOM0_1_TOUT1_P02_1_OUT, IfxGtm_ATOM0_2_TOUT2_P02_2_OUT};

    // init
    // - clock
    IfxGtm_enable(&MODULE_GTM);                                     /* Enable GTM                                   */
    IfxGtm_Cmu_enableClocks(&MODULE_GTM, IFXGTM_CMU_CLKEN_FXCLK);   /* Enable the FXU clock                         */

    // - timer
    IfxGtm_Atom_Timer timer;
    IfxGtm_Atom_Timer_Config timerConfig;
    IfxGtm_Atom_Timer_initConfig(&timerConfig, &MODULE_GTM);

//    g_atomConfig.pin.outputPin = &LED;                                  /* Set LED as output                        */
//    g_atomConfig.synchronousUpdateEnabled = TRUE;                       /* Enable synchronous update                */
    timerConfig.atom = ccx[0]->atom;
    timerConfig.timerChannel = ccx[0]->channel;  // but the channel of coutx[0] is also used! How to tell the driver?
//    timerConfig.initPins = FALSE;  // No idea what to use
    timerConfig.base.frequency              = PWM_FREQUENCY;
    timerConfig.base.countDir               = IfxStdIf_Timer_CountDir_up;
//    timerConfig.base.trigger.enabled        = TRUE;  // 1/2 IfxCpu_Trap_busError
//    timerConfig.base.trigger.outputEnabled  = TRUE;  // 2/2 IfxCpu_Trap_busError

//    IfxGtm_Atom_Timer_init(&timer, &timerConfig);
//    timerConfig.base.trigger.outputEnabled      = TRUE;
//    timerConfig.base.trigger.enabled            = TRUE;
//    timerConfig.base.trigger.triggerPoint       = 100;
//    timerConfig.base.trigger.risingEdgeAtPeriod = TRUE;

    IfxGtm_Atom_Timer_init(&timer, &timerConfig);
    IfxGtm_Atom_Timer_run(&timer);

    // - pwmHl
//    IfxGtm_AtomDtm_PwmHl            pwmHl;
//    IfxGtm_Atom_Dtm_PwmHl_Config    pwmHlConfig;
//    IfxGtm_Atom_Dtm_PwmHl_initConfig(&pwmHlConfig);
    IfxGtm_Atom_PwmHl               pwmHl;
    IfxGtm_Atom_PwmHl_Config        pwmHlConfig;
    IfxGtm_Atom_PwmHl_initConfig(&pwmHlConfig);

//    pwmHlConfig.base.deadtime           = 200e-9;
//    pwmHlConfig.base.minPulse           = 100e-9;
    pwmHlConfig.base.channelCount       = NUMBER_OF_CHANNELS;
//    pwmHlConfig.base.emergencyEnabled   = FALSE;
    pwmHlConfig.base.outputMode         = IfxPort_OutputMode_pushPull;
    pwmHlConfig.base.outputDriver       = IfxPort_PadDriver_cmosAutomotiveSpeed3;
    pwmHlConfig.base.ccxOutputEnabled   = TRUE;
    pwmHlConfig.base.coutxOutputEnabled = TRUE;
//    pwmHlConfig.base.ccxActiveState     = Ifx_ActiveState_high;
//    pwmHlConfig.base.coutxActiveState   = Ifx_ActiveState_high;

    // ccx and coutx configuration moved upwards

    pwmHlConfig.timer = &timer;
    pwmHlConfig.atom = timer.atomIndex;
    pwmHlConfig.ccx = ccx;
    pwmHlConfig.coutx = coutx;
//    IfxGtm_Atom_Dtm_PwmHl_init(&pwmHl, &pwmHlConfig);
//    IfxGtm_Atom_Dtm_PwmHl_setMode(&pwmHl, Ifx_Pwm_Mode_leftAligned);
    IfxGtm_Atom_PwmHl_init(&pwmHl, &pwmHlConfig);
    IfxGtm_Atom_PwmHl_setMode(&pwmHl, Ifx_Pwm_Mode_leftAligned);

    // run
    IfxGtm_Atom_Timer_run(&timer);

    IfxGtm_Atom_Timer_disableUpdate(&timer);
    IfxGtm_Atom_Timer_setSingleMode(&timer, FALSE);
    IfxGtm_Atom_Timer_applyUpdate(&timer);

    Ifx_TimerValue tOn = (uint32) PWM_PERIOD/2;
//    Ifx_TimerValue tOn[2] = {PWM_PERIOD/2, PWM_PERIOD/2};
    IfxGtm_Atom_PwmHl_setOnTime(&pwmHl, &tOn);


//    // - setup stdif
//    IfxStdIf_Timer timer_stdif;
//    IfxGtm_Atom_Timer_stdIfTimerInit(&timer_stdif, &timer);
//    //    IfxStdIf_Timer *timer_stdif = IfxStdIf_PwmHl_getTimer(&pwmHl_stdif);
//
//    IfxStdIf_PwmHl pwmHl_stdif;
//    IfxGtm_Atom_Dtm_PwmHl_stdIfPwmHlInit(&pwmHl_stdif, &pwmHl);
//
//    // - run stdif
//    IfxStdIf_Timer_run(&timer_stdif);
//
//    IfxStdIf_Timer_disableUpdate(&timer_stdif);  // To allow timer changes
//
//    Ifx_TimerValue tOn = (uint32) PWM_PERIOD/2;
//
//    // Set trigger point
////    IfxStdIf_Timer_setTrigger(&timer_stdif, tOn);
//
//    // Set period of cycle
//    Ifx_TimerValue period = (uint32) PWM_PERIOD;
//    IfxStdIf_Timer_setPeriod(&timer_stdif, period);
//
//    // Set duty cycle on time
//    IfxStdIf_PwmHl_setOnTime(&pwmHl_stdif, &tOn);
//
//    IfxStdIf_Timer_applyUpdate(&timer_stdif);  // To implement timer changes
//    IfxStdIf_Timer_run(&timer_stdif);
//
//
//    // - Diagnostics to test if changes work
//    Ifx_Pwm_Mode current_mode;
//    current_mode = IfxStdIf_PwmHl_getMode(&pwmHl_stdif);
//    current_mode = IfxStdIf_PwmHl_getMode(&pwmHl_stdif);
//
//    float32 frequencyGet = IfxStdIf_Timer_getFrequency(&timer_stdif);
//
//    float32 ten = (float32) 100 + frequencyGet;
//
//    IfxStdIf_Timer_setFrequency(&timer_stdif, ten);
}

